@@ -6,10 +6,11 @@ if [ $(whoami) != "root" ]; then
     exit 1
 fi
 
-echo "Starting mysql server"
-/etc/init.d/mariadb start
+echo "Starting MariaDB server"
+chmod u+rwx /etc/init.d/mysql
+/etc/init.d/mysql start
 
-echo "Waiting for mysql to start"
+echo "Waiting for MariaDB to start"
 while ! mysqladmin ping -h 127.0.0.1 --silent; do
     sleep 0.2
 done
