@@ -11,11 +11,13 @@ else:
     compat.register()
     del compat
 
+import shapely
 from packaging.version import parse as parse_version
 from shapely.geometry import LineString
 from shapely.geometry import Point
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import String
@@ -54,6 +56,7 @@ from . import skip_postgis1
 from . import test_only_with_dialects
 
 SQLA_LT_2 = parse_version(SA_VERSION) <= parse_version("1.999")
+SHAPELY_LT_21 = parse_version(shapely.__version__) < parse_version("2.1")
 
 
 class TestAdmin:
@@ -229,6 +232,105 @@ class TestAdmin:
 
         assert marks == ["before_create", "after_create", "before_drop", "after_drop"]
 
+    def test_computed_column_core(self, conn):
+        meta = MetaData()
+
+        # Define the table
+        t = Table(
+            "computed_column",
+            meta,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            Column("y", Integer),
+            Column(
+                "computed_geom",
+                Geometry(geometry_type="POINT", srid=4326),
+                Computed("ST_POINT(x, y)", persisted=True),
+            ),
+        )
+
+        # Create the table
+        meta.create_all(bind=conn)
+
+        conn.execute(
+            t.insert(),
+            [
+                {"x": "1", "y": "2"},
+                {"x": "2", "y": "3"},
+            ],
+        )
+
+        res = conn.execute(
+            select(
+                [
+                    t.c.id,
+                    t.c.x,
+                    t.c.y,
+                    t.c.computed_geom.ST_AsText(),
+                ]
+            ),
+        ).fetchall()
+
+        # Check inserted data
+        for i in res:
+            x = i[1]
+            y = i[2]
+            p_x, p_y = [int(j) for j in re.findall(r"\d+", i[3])]
+            assert x == p_x
+            assert y == p_y
+
+        # Drop the table
+        meta.drop_all(bind=conn)
+
+    def test_computed_column_orm(self, conn, base, metadata):
+
+        # Define the table
+        class ComputedGeomTable(base):
+            __tablename__ = "computed_column"
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+            y = Column(Integer)
+            computed_geom = Column(
+                Geometry(geometry_type="POINT", srid=4326),
+                Computed("ST_POINT(x, y)", persisted=True),
+            )
+
+            def __init__(self, computed_geom):
+                self.computed_geom = computed_geom
+
+        # Create the table
+        metadata.create_all(bind=conn)
+
+        conn.execute(
+            ComputedGeomTable.__table__.insert(),
+            [
+                {"x": "1", "y": "2"},
+                {"x": "2", "y": "3"},
+            ],
+        )
+
+        res = conn.execute(
+            select(
+                [
+                    ComputedGeomTable.__table__.c.id,
+                    ComputedGeomTable.__table__.c.x,
+                    ComputedGeomTable.__table__.c.y,
+                    ComputedGeomTable.__table__.c.computed_geom.ST_AsText(),
+                ]
+            ),
+        ).fetchall()
+
+        # Check inserted data
+        for i in res:
+            x = i[1]
+            y = i[2]
+            p_x, p_y = [int(j) for j in re.findall(r"\d+", i[3])]
+            assert x == p_x
+            assert y == p_y
+
+        # Drop the table
+        metadata.drop_all(bind=conn)
+
 
 class TestInsertionCore:
     def test_insert(self, conn, Lake, setup_tables):
@@ -237,27 +339,30 @@ class TestInsertionCore:
         conn.execute(
             Lake.__table__.insert(),
             [
-                {"geom": "SRID=4326;LINESTRING(0 0,1 1)"},
-                {"geom": WKTElement("LINESTRING(0 0,2 2)", srid=4326)},
-                {"geom": WKTElement("SRID=4326;LINESTRING(0 0,2 2)", extended=True)},
-                {"geom": from_shape(LineString([[0, 0], [3, 3]]), srid=4326)},
+                {"id": 1, "geom": "SRID=4326;LINESTRING(0 0,1 1)"},
+                {"id": 2, "geom": WKTElement("LINESTRING(0 0,2 2)", srid=4326)},
+                {"id": 3, "geom": WKTElement("SRID=4326;LINESTRING(0 0,2 2)", extended=True)},
+                {"id": 4, "geom": from_shape(LineString([[0, 0], [3, 3]]), srid=4326)},
             ],
         )
 
-        results = conn.execute(Lake.__table__.select())
+        results = conn.execute(Lake.__table__.select().order_by("id"))
         rows = results.fetchall()
 
         row = rows[0]
         assert isinstance(row[1], WKBElement)
         wkt = conn.execute(from_shape(LineString([[0, 0], [3, 3]]), srid=4326).ST_AsText()).scalar()
-        wkt = conn.execute(row[1].ST_AsText()).scalar()
+        q1 = row[1].ST_AsText()
+        wkt = conn.execute(q1).scalar()
         assert format_wkt(wkt) == "LINESTRING(0 0,1 1)"
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
 
         row = rows[1]
         assert isinstance(row[1], WKBElement)
-        wkt = conn.execute(row[1].ST_AsText()).scalar()
+        q2 = row[1].ST_AsText()
+        wkt = conn.execute(q2).scalar()
+
         assert format_wkt(wkt) == "LINESTRING(0 0,2 2)"
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
@@ -391,6 +496,9 @@ class TestInsertionCore:
             inserted_elements.append({"geom": wkb_elem})
             inserted_elements.append({"geom": wkb_elem.as_ewkb()})
 
+        print(inserted_elements)
+        # raise ValueError()
+
         # Insert the elements
         conn.execute(
             GeomTypeTable.__table__.insert(),
@@ -416,13 +524,76 @@ class TestInsertionCore:
                 # Some dialects return MULTIPOINT geometries with nested parenthesis and others
                 # do not so we remove them before checking the results
                 checked_wkt = re.sub(r"\(([0-9\.]+)\)", "\\1", checked_wkt)
-            if row_id >= 5 and dialect_name in ["geopackage"] and has_m:
+            if row_id >= 5 and dialect_name in ["geopackage"] and has_m and SHAPELY_LT_21:
                 # Currently Shapely does not support geometry types with M dimension
                 assert checked_wkt != expected_wkt
             else:
                 assert checked_wkt == expected_wkt
             assert srid == 4326
 
+    @pytest.mark.parametrize(
+        "geom_type",
+        [
+            pytest.param("POINT", id="Point"),
+            pytest.param("POINTZ", id="Point Z"),
+            pytest.param("POINTM", id="Point M"),
+            pytest.param("POINTZM", id="Point ZM"),
+            pytest.param("LINESTRING", id="LineString"),
+            pytest.param("LINESTRINGZ", id="LineString Z"),
+            pytest.param("LINESTRINGM", id="LineString M"),
+            pytest.param("LINESTRINGZM", id="LineString ZM"),
+            pytest.param("POLYGON", id="Polygon"),
+            pytest.param("POLYGONZ", id="Polygon Z"),
+            pytest.param("POLYGONM", id="Polygon M"),
+            pytest.param("POLYGONZM", id="Polygon ZM"),
+            pytest.param("MULTIPOINT", id="Multi Point"),
+            pytest.param("MULTIPOINTZ", id="Multi Point Z"),
+            pytest.param("MULTIPOINTM", id="Multi Point M"),
+            pytest.param("MULTIPOINTZM", id="Multi Point ZM"),
+            pytest.param("MULTILINESTRING", id="Multi LineString"),
+            pytest.param("MULTILINESTRINGZ", id="Multi LineString Z"),
+            pytest.param("MULTILINESTRINGM", id="Multi LineString M"),
+            pytest.param("MULTILINESTRINGZM", id="Multi LineString ZM"),
+            pytest.param("MULTIPOLYGON", id="Multi Polygon"),
+            pytest.param("MULTIPOLYGONZ", id="Multi Polygon Z"),
+            pytest.param("MULTIPOLYGONM", id="Multi Polygon M"),
+            pytest.param("MULTIPOLYGONZM", id="Multi Polygon ZM"),
+        ],
+    )
+    @pytest.mark.parametrize(
+        "dimension",
+        [
+            pytest.param(
+                None,
+                id="Dimension is None",
+            ),
+            pytest.param(
+                -1,
+                id="Negative dimension",
+            ),
+            pytest.param(
+                1,
+                id="Wrong dimension",
+            ),
+        ],
+    )
+    def test_check_ctor_args(self, dialect_name, base, conn, metadata, geom_type, dimension):
+        ndims = 2
+        if "Z" in geom_type[-2:]:
+            ndims += 1
+        if geom_type.endswith("M"):
+            ndims += 1
+
+        if ndims > 2 and dialect_name in ["mysql", "mariadb"]:
+            # Explicitly skip MySQL dialect to show that it can only work with 2D geometries
+            pytest.xfail(reason="MySQL only supports 2D geometry types")
+
+        if dimension is not None:
+            with pytest.raises(ValueError):
+                Geometry(srid=4326, geometry_type=geom_type, dimension=dimension)
+        else:
+            Geometry(srid=4326, geometry_type=geom_type, dimension=dimension)
+
     @test_only_with_dialects("postgresql", "sqlite")
     def test_insert_geom_poi(self, conn, Poi, setup_tables):
         conn.execute(
@@ -1059,6 +1230,31 @@ class TestReflection:
                 assert type_.srid == 4326
                 assert type_.dimension == 4
 
+            if dialect_name == "postgresql":
+                type_ = t.c.geom_geog.type
+                assert isinstance(type_, Geography)
+                assert type_.geometry_type == "LINESTRING"
+                assert type_.srid == 4326
+                assert type_.dimension == 2
+
+                type_ = t.c.geom_geog_no_idx.type
+                assert isinstance(type_, Geography)
+                assert type_.geometry_type == "LINESTRING"
+                assert type_.srid == 4326
+                assert type_.dimension == 2
+
+                type_ = t.c.rast.type
+                assert isinstance(type_, Raster)
+                assert type_.geometry_type is None
+                assert type_.srid == -1
+                assert type_.dimension is None
+
+                type_ = t.c.rast_no_idx.type
+                assert isinstance(type_, Raster)
+                assert type_.geometry_type is None
+                assert type_.srid == -1
+                assert type_.dimension is None
+
         # Drop the table
         t.drop(bind=conn)
 
@@ -1110,6 +1306,10 @@ class TestReflection:
                         "idx_lake_geom",
                         "CREATE INDEX idx_lake_geom ON gis.lake USING gist (geom)",
                     ),
+                    (
+                        "idx_lake_geom_geog",
+                        "CREATE INDEX idx_lake_geom_geog ON gis.lake USING gist (geom_geog)",
+                    ),
                     (
                         "idx_lake_geom_m",
                         "CREATE INDEX idx_lake_geom_m ON gis.lake USING gist (geom_m)",
@@ -1122,6 +1322,10 @@ class TestReflection:
                         "idx_lake_geom_zm",
                         "CREATE INDEX idx_lake_geom_zm ON gis.lake USING gist (geom_zm)",
                     ),
+                    (
+                        "idx_lake_rast",
+                        "CREATE INDEX idx_lake_rast ON gis.lake USING gist (st_convexhull(rast))",
+                    ),
                     (
                         "lake_pkey",
                         "CREATE UNIQUE INDEX lake_pkey ON gis.lake USING btree (id)",
@@ -1261,3 +1465,23 @@ class TestAsBinaryWKT:
 
         # Drop the table
         t.drop(bind=conn)
+
+
+class TestCompileQuery:
+    def test_compile_query(self, conn):
+        wkb = b"\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@"
+        elem = WKBElement(wkb)
+        query = select([func.ST_AsText(elem)])
+        compiled_with_literal = str(query.compile(conn, compile_kwargs={"literal_binds": True}))
+        res_text = conn.execute(text(compiled_with_literal)).scalar()
+        assert res_text == "POINT(1 2)"
+
+        compiled_without_literal = str(query.compile(conn, compile_kwargs={"literal_binds": False}))
+
+        res_query = conn.execute(query).scalar()
+        assert res_query == "POINT(1 2)"
+
+        assert compiled_with_literal.startswith("SELECT ST_AsText(")
+        assert "0101000000000000000000f03f0000000000000040" in compiled_with_literal
+        assert compiled_without_literal.startswith("SELECT ST_AsText(")
+        assert "0101000000000000000000f03f0000000000000040" not in compiled_without_literal
